﻿using System;
using System.Linq;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Data;
using Word = Microsoft.Office.Interop.Word;

namespace EmbeddedLinkUpdater.Code.Documents
{
    public class WordDocument : Document
    {
        public WordDocument(string fileName) : base(fileName)
        {
        }

        public override void UpdateDocumentLinks(DataSet linksToReplace)
        {
            AnalyzeOrUpdateLinks(linksToReplace, true);
        }

        public override List<string> AnalyzeLinks()
        {
            AnalyzeOrUpdateLinks(null, false);

            return DocumentLinks;
        }

        private void AnalyzeOrUpdateLinks(DataSet linksToReplace, bool update)
        {
            Word.Application wordApplication = null;

            try
            {
                wordApplication = new Word.Application();
                wordApplication.AutomationSecurity = Microsoft.Office.Core.MsoAutomationSecurity.msoAutomationSecurityForceDisable;
         
                if (FileInUse(FileName))
                    throw new ApplicationException("Failed to update. File in use.");

                var document = wordApplication.Documents.Open(FileName, ReadOnly: false, Visible: false);

                document.Activate();

                foreach (var link in from Word.Range range in document.StoryRanges from Word.Hyperlink link in range.Hyperlinks select link)
                {
                    if (update && linksToReplace != null)
                    {
                        var tableLinksToReplace = linksToReplace.Tables[0];
                        foreach (DataRow row in tableLinksToReplace.Rows)
                        {
                            if (link.Address != null)
                                link.Address = link.Address.ReplaceIgnoreCase(row["OldLink"].ToString(), row["NewLink"].ToString());

                            if (!String.IsNullOrEmpty(link.TextToDisplay))
                                link.TextToDisplay = link.TextToDisplay.ReplaceIgnoreCase(row["OldLink"].ToString(), row["NewLink"].ToString());
                        }
                    }
                    else
                    {
                        DocumentLinks.Add(link.Address);
                    }

                    Marshal.ReleaseComObject(link);
                }

                document.Close(update);
                Marshal.ReleaseComObject(document);
            }
            catch (ApplicationException)
            {
                throw;
            }
            catch (Exception ex)
            {
                throw new ApplicationException("Failed to update document links", ex);
            }
            finally
            {
                if (wordApplication != null)
                {
                    wordApplication.Quit();
                    Marshal.ReleaseComObject(wordApplication);
                }

                GC.Collect();
                GC.WaitForPendingFinalizers();
            }
        }
    }
}
